## Base support for new device

Add GTIN/dSID et al. to the product_tables in dsdata

# Enable HardwareType lookup

Add mapping from ProductId to HardwareType

Device::calculateHWInfo:
```
  m_hardwareType = [&]() -> HardwareType {
    ...
        {{1, 0x08C9}, HardwareType::GE_SDM200JS}, //
```

ProductId is function of device type and model number

```
  TKM210: ProductID = DEVICE_TYPE_TKM << 10 | 210
```


# Introduce new HardwareType lookup

src/model/modelconst.h;
```
  DS_REFLECTED_ENUM((dss), HardwareType,
                    (SW_SKM300)   //
```


# Declare device input/output features:
-> note compiler warnings in switch/case statements

inputs
```
  static std::vector<ChannelType> hardwareOutputChannels(HardwareType hardwareType, const Device& device) {
  ...
```

*  bool StructureRequestHandler::onlyOneGroupMembership(const Device& dev) const

outputs:
```
@@ -4416,6 +4421,7 @@ std::vector<DeviceAction> Device::actions() const {
```

if the device has output then you also need to update

* uint8_t Device::getSWThresholdAddress() const
* dsModel::ShadeType shadeType(const Device& device)

# Describe advanced device features


see model-features
```
  const ModelFeatureId MF_SW_SK20[] =
  {
      ModelFeatureId::temperatureoffset,
      ModelFeatureId::identification,
  };
```

# 2way pairing

* bool Device::is2WayMaster() const {
* bool Device::is2WaySlave() const {

# Icon support

Place icons into folder data/images and add its name to data/image.list Makefile

If the device is a joker device, you need to add an icon for each color it can incorporate.

```
void Device::updateIconPath() {
```
only declare the base name of the icon file:

```
    } else if ((deviceType == DEVICE_TYPE_KM) && (getDeviceNumber() == 300)) {
      m_iconPath = "km300";
```
the function then appends the devices static color as suffix

```
    m_iconPath += "_" + getColorString(deviceClass);
```

for joker devices the current color of the current configuration is added as well

```
    if (getDeviceClass() == DEVICE_CLASS_SW) {
      ...
      m_iconPath += "_" + getColorString(jokerColor);
    }
```

hence the final icon file name for a regular and a joker device is:

* tkm300_yellow.png
* skm300_black_yellow.png


# Create device mockup
you probably already noticed the warning in the switch/case

see src/model/device-mock.cpp
```
    case HardwareType::SW_SKM300:
      dev->setPartiallyFromSpec(
          DeviceSpec{.FunctionID = functionID105(DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_4),
                     .ProductID = DEVICE_TYPE_SKM << 10 | 300,
                     .revisionId = revisionId,
                     .activeGroup = static_cast<int>(ApplicationType::lights)}); // default group
      break;
```

As a checklist for completeness, implement these tests

```
  GIVEN("SW-SKM300") {
    auto dev = createDevice(apartment, HardwareType::SW_SKM300);
    CHECK(dev->getProductID() == 0x392c);
    CHECK(dev->getFunctionID() == 0x81cb);
    CHECK(dev->getDeviceClass() == DEVICE_CLASS_SW);
    CHECK(dev->getDeviceType() == DEVICE_TYPE_SKM);
    CHECK(dev->hasOutput() == false);
    CHECK(dev->hasInput() == true);
    CHECK(dev->getIconPath() == "skm300_black_yellow.png"); // default
    CHECK(shadeType(*dev) == dsModel::ShadeType{});         // input only, does not apply
    CHECK_THROWS(dev->getSWThresholdAddress());             // input only, not used
  }
```

see also:

git show 8edd48b5bb765d3b6c5a4d121bf56d101c545eb4
