#!/bin/bash
set -e

DOCKER_DIR="$(dirname $(realpath $0))"
. ${DOCKER_DIR}/env.sh

TEMP=`getopt -o h --long configure,disable-scripting -- "$@"`

# Replace arguments by preprocessed arguments
eval set -- "$TEMP"

TRUE=0
FALSE=1

# /usr/lib/ccache contains links for most common compiler, e.g. gcc, clang
# -> you can still select the compiler via CC/CXX
export PATH=/usr/lib/ccache:$PATH

printf "CC : %s\n" $(which ${CC:-gcc})
printf "CXX: %s\n" $(which ${CXX:-g++})

CONFIGURE=$FALSE
SCRIPTING=
while true; do
    case "$1" in
        --configure) CONFIGURE=$TRUE; shift;;
        --disable-scripting) SCRIPTING=$FALSE; shift;;
        --) shift; break;;
        *) break;;
    esac
done

if [ $# -ne 0 ]; then
    echo "[ERROR] unkown argument: $@"
    exit 1
fi

# building out of tree, these files n the source tree confuse make
rm -f src/messages/*.pb.*

if [ $CONFIGURE -eq $TRUE ]; then
  echo do configure
  autoreconf -i

  mkdir -p build-autoconf
  cd build-autoconf

  ../configure --prefix=$PREFIX --with-search=$PREFIX \
    ${SCRIPTING:+--disable-scripting} \
    --enable-debug \
    --enable-stack-protect \
    --enable-asan \
    --enable-insecure-cookie
    # --enable-tsan/--enable-asan are exclusive or
else
  cd build-autoconf
fi

nice make -j $(nproc)
#make install
