/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds-api/event.h>
#include <ds-net/signalr.h>
#include <ds/asio/resource.h>

namespace dsApi {

struct NotificationsSessionHooks {
  dsNet::BufferedMessageWriterConfig bufferedMessageWriterConfig{};
};
struct NotificationsSession : virtual dsNet::ISession {
  virtual void asyncRun(NotificationsSessionHooks&&) = 0;
};
dsNet::ClosePtr<NotificationsSession> makeNotificationsSession(dsNet::Sessions& sessions,
                                                               dsNet::ClosePtr<dsNet::SignalrSession>&&,
                                                               dsApi::EventHub& eventHub);

struct NotificationsConfig {
  dsNet::BufferedMessageWriterConfig sessionWriteBuffer =
      dsNet::BufferedMessageWriterConfig{.countCapacity = 256, .sizeCapacity = 256 * 1024};
};
struct Notifications {
  virtual ~Notifications() = default;
};

std::unique_ptr<Notifications> makeNotifications(ds::asio::ResourceBase& parent,
                                                 std::unique_ptr<dsNet::SignalrAcceptor>, dsApi::EventHub&,
                                                 NotificationsConfig&& = {});

} // namespace dsApi

DS_JSON_DECLARE_STRUCT((dsApi), NotificationsConfig, (sessionWriteBuffer));
