/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "loop.h"

#include <ds/asio/catch.h>
#include <ds/log.h>
#include <ds/system-mock.h>
#include <sys/prctl.h>

#include <future>

#include "loop-impl.h"

static const char* TAGS = "[dsAsioLoop][dsAsio][ds]";
using namespace std::chrono_literals;

SCENARIO("dsAsioLoopImpl", TAGS) {
  const auto mainThreadId = std::this_thread::get_id();
  auto loop = ds::asio::LoopImpl{.name = "dsAsioLoopTest"};

  SECTION("executes handlers in another threas") {
    auto task = std::packaged_task<void(void)>([&] { CHECK(mainThreadId != std::this_thread::get_id()); });
    loop.io.post([&] { task(); });
    task.get_future().get();
  }

  SECTION("exceptions are swallowed") {
    loop.io.post([&] { DS_FAIL_REQUIRE("test"); });

    auto task = std::packaged_task<void(void)>([&] { CHECK(mainThreadId != std::this_thread::get_id()); });
    loop.io.post([&] { task(); });
    task.get_future().get();
  }

  SECTION("set linux thread name") {
    char name[17] = {0};
    auto task = std::packaged_task<void(void)>([&] { prctl(PR_GET_NAME, name, 0L, 0L, 0L); });
    loop.io.post([&] { task(); });
    task.get_future().get();
    REQUIRE(loop.name == std::string_view{name});
  }
}

SCENARIO("dsAsioLoop", TAGS) {
  auto system = ds::SystemMock{};
  const auto mainThreadId = std::this_thread::get_id();
  auto loop = ds::asio::Loop{system.conf(), "testLoop", {}};

  SECTION("Context has different thread") { CHECK(loop.context().threadId != mainThreadId); }

  SECTION("invoke synchronizes to thread") {
    CHECK(loop.context().threadId == loop.invoke([&] { return std::this_thread::get_id(); }));
  }
}
