/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "thread-pool.h"
#include <ds/asio/catch.h>
#include <ds/log.h>
#include <future>
#include <mutex>

static const char* TAGS = "[dsAsioThreadPool][dsAsio][ds]";
using namespace std::chrono_literals;

SCENARIO("dsAsioThreadPool", TAGS) {
  GIVEN("0 threads") {
    ds::asio::ThreadPool threadPool{0};
    THEN("can be created and destroyed") {}

    THEN("tasks are not executed") {
      std::atomic_int executed{0};
      threadPool.post([&] { executed++; });
      std::this_thread::sleep_for(1ms);
      CHECK(executed == 0);
    }
  }

  GIVEN("2 threads") {
    ds::asio::ThreadPool threadPool{2};
    THEN("can be created and destroyed") {}

    THEN("up to 2 tasks from 4 are running in parallel") {
      std::mutex mutex;
      int parallel{0};
      int maxParallel{0};
      int toFinish{4};
      for (int i = 0; i < 4; i++) {
        threadPool.post([&parallel, &maxParallel, &mutex, &toFinish]() mutable {
          {
            std::lock_guard<std::mutex> lock(mutex);
            parallel++;
            maxParallel = std::max(parallel, maxParallel);
          }
          while (1) {
            std::this_thread::sleep_for(1ms);
            std::lock_guard<std::mutex> lock(mutex);
            if (toFinish <= 2 || (toFinish > 2 && parallel == 2)) {
              parallel--;
              toFinish--;
              break;
            }
          }
        });
      }
      while (1) {
        std::this_thread::sleep_for(1ms);
        std::lock_guard<std::mutex> lock(mutex);
        if (toFinish == 0) {
          break;
        }
      }
      CHECK(maxParallel == 2);
    }

    THEN("exceptions are swallowed") {
      threadPool.post([&] { DS_FAIL_REQUIRE("test"); });

      auto task = std::packaged_task<void(void)>([] {});
      threadPool.post([&] { task(); });
      task.get_future().get();
    }
  }
}
