/*
 *  Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <ds/common.h>
#include <gsl.h>
#include <chrono>
#include "str.h"

namespace ds {

/// Alias to system time point.
///
/// Commonly used in serialized data structures.
/// Equivalent to javascript `Date`.
typedef std::chrono::system_clock::time_point DateTime;

/// Formats DateTime to ISO8601 string in local time zone with milliseconds precision.
std::string toLocalMsIsoString(DateTime);

/// Formats DateTime to ISO8601 string in UTC in milliseconds.
std::string toString(DateTime);

/// Parses ISO8601 string to DateTime.
DateTime toDateTime(gsl::cstring_span s);

/// like std::max, but for ds::DateTime.
inline DateTime max(ds::DateTime a, ds::DateTime b) {
  return a > b ? a : b;
};

namespace detail {
template <>
struct Str<DateTime> {
  static void f(std::ostream &ostream, const DateTime &x) { ostream << toString(x); }
};
} // namespace detail

} // namespace ds
