/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "backend.h"
#include <ds/catch/catch.h>
#include <ds/log.h>
#include <boost/locale/generator.hpp>
#include <boost/locale/message.hpp>

static ds::LogChannel dsLogChannel("dsLocaleBackendTest");

static const char* TAGS = "[dsLocale][ds]";

namespace {

TEST_CASE("dsLocaleBackend", TAGS) {
  boost::locale::generator generator(ds::locale::backend());
  // generator.set_default_messages_domain(...) is buggy and crashes on boost_1.5.8
  generator.add_messages_domain("ds-locale-messages-test");
  // DS_LOCALE_SRC_DIR is passed as -D... by build system
  generator.add_messages_path(ds::str(DS_LOCALE_SRC_DIR, "/test"));
  auto locale = generator("de_DE");

  SECTION("integration with boost::locale::translate works") {
    CHECK(boost::locale::translate("welcomeId").str(locale) == "Willkommen");
    CHECK(boost::locale::translate("context1", "welcomeId").str(locale) == "Willkommen1");
    CHECK(boost::locale::translate("context2", "welcomeId").str(locale) == "Willkommen2");
    CHECK(boost::locale::translate("itemId", "itemIds", 1).str(locale) == "Stück");
    CHECK(boost::locale::translate("itemId", "itemIds", 2).str(locale) == "Stücke");
    CHECK(boost::locale::translate("context1", "itemId", "itemIds", 1).str(locale) == "Stück1");
    CHECK(boost::locale::translate("context1", "itemId", "itemIds", 2).str(locale) == "Stücke1");
    CHECK(boost::locale::translate("context2", "itemId", "itemIds", 1).str(locale) == "Stück2");
    CHECK(boost::locale::translate("context2", "itemId", "itemIds", 2).str(locale) == "Stücke2");

    std::string domain = "ds-locale-messages-test1-en,ds-locale-messages-test2-de";
    CHECK(boost::locale::translate("welcomeId").str(locale, domain) == "welcomeId");
    CHECK(boost::locale::translate("context1", "welcomeId").str(locale, domain) == "Welcome1-1");
    CHECK(boost::locale::translate("context2", "welcomeId").str(locale, domain) == "Willkommen2-2");
    CHECK(boost::locale::translate("itemId", "itemIds", 1).str(locale, domain) == "itemId");
    CHECK(boost::locale::translate("itemId", "itemIds", 2).str(locale, domain) == "itemIds");
    CHECK(boost::locale::translate("context1", "itemId", "itemIds", 1).str(locale, domain) == "Item1-1");
    CHECK(boost::locale::translate("context1", "itemId", "itemIds", 2).str(locale, domain) == "Items1-1");
    CHECK(boost::locale::translate("context2", "itemId", "itemIds", 1).str(locale, domain) == "Stück2-2");
    CHECK(boost::locale::translate("context2", "itemId", "itemIds", 2).str(locale, domain) == "Stücke2-2");
  }
}

} // namespace
