/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include "event-logger.h"

#include <ds/catch/catch.h>
#include <ds/log.h>

static ds::LogChannel dsLogChannel("handlerEventLoggerTest");
static const char* TAGS = "[handler][event-logger]";

using namespace dss;
using namespace std::literals;
using namespace trompeloeil;
using trompeloeil::_;

namespace {

TEST_CASE("SystemEventLog", TAGS) {
  SystemEventLog systemEventLog(Event{});

  SECTION("CallOrigin conversion") {
    SECTION("domain") {
      struct X {
        CallOrigin callOrigin;
        std::string printable;
      };
      const auto& x = GENERATE(values(std::initializer_list<X>{{CallOrigin::unknown, "(unspecified)"s},
                                                               {CallOrigin::scripting, "Scripting"s},
                                                               {CallOrigin::json, "JSON"s},
                                                               {CallOrigin::subscription, "Bus-Handler"s},
                                                               {CallOrigin::test, "Test"s},
                                                               {CallOrigin::system, "System"s},
                                                               {CallOrigin::binaryInput, "System-Binary-Input"s},
                                                               {CallOrigin::dsmApi, "dSM-API"s},
                                                               {CallOrigin::systemStartup, "10"s}}));
      INFO(ds::str(x.callOrigin));
      CHECK(systemEventLog.getCallOrigin(x.callOrigin) == x.printable);
    }
  }
}

} // namespace
