/*
    Copyright (c) 2021 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#include "buttonattributes.h"

using namespace dss;
using namespace dss::model;

std::optional<dss::model::AutomationInputMode>
DSToEnum<dss::model::AutomationInputMode, dss::ButtonInputMode, void>::tryTo(dss::ButtonInputMode x) {
  switch (x) {
    case ButtonInputMode::AKM_STANDARD: return AutomationInputMode::standard;
    case ButtonInputMode::AKM_INVERTED: return AutomationInputMode::inverted;
    case ButtonInputMode::AKM_ON_RISING_EDGE: return AutomationInputMode::on_rising_edge;
    case ButtonInputMode::AKM_ON_FALLING_EDGE: return AutomationInputMode::on_falling_edge;
    case ButtonInputMode::AKM_OFF_RISING_EDGE: return AutomationInputMode::off_rising_edge;
    case ButtonInputMode::AKM_OFF_FALLING_EDGE: return AutomationInputMode::off_falling_edge;
    case ButtonInputMode::AKM_RISING_EDGE: return AutomationInputMode::rising_edge;
    case ButtonInputMode::AKM_FALLING_EDGE: return AutomationInputMode::falling_edge;
    case ButtonInputMode::STANDARD:
    case ButtonInputMode::TURBO:
    case ButtonInputMode::SWITCHED:
    case ButtonInputMode::TWO_WAY_DW_WITH_INPUT1:
    case ButtonInputMode::TWO_WAY_DW_WITH_INPUT2:
    case ButtonInputMode::TWO_WAY_DW_WITH_INPUT3:
    case ButtonInputMode::TWO_WAY_DW_WITH_INPUT4:
    case ButtonInputMode::TWO_WAY_UP_WITH_INPUT1:
    case ButtonInputMode::TWO_WAY_UP_WITH_INPUT2:
    case ButtonInputMode::TWO_WAY_UP_WITH_INPUT3:
    case ButtonInputMode::TWO_WAY_UP_WITH_INPUT4:
    case ButtonInputMode::TWO_WAY:
    case ButtonInputMode::ONE_WAY:
    case ButtonInputMode::HEATING_PUSHBUTTON:
    case ButtonInputMode::DEACTIVATED: return {};
  }
  return {};
}

std::optional<dss::ButtonInputMode> DSToEnum<dss::ButtonInputMode, dss::model::AutomationInputMode, void>::tryTo(
    dss::model::AutomationInputMode x) {
  switch (x) {
    case AutomationInputMode::standard: return ButtonInputMode::AKM_STANDARD;
    case AutomationInputMode::inverted: return ButtonInputMode::AKM_INVERTED;
    case AutomationInputMode::on_rising_edge: return ButtonInputMode::AKM_ON_RISING_EDGE;
    case AutomationInputMode::on_falling_edge: return ButtonInputMode::AKM_ON_FALLING_EDGE;
    case AutomationInputMode::off_rising_edge: return ButtonInputMode::AKM_OFF_RISING_EDGE;
    case AutomationInputMode::off_falling_edge: return ButtonInputMode::AKM_OFF_FALLING_EDGE;
    case AutomationInputMode::rising_edge: return ButtonInputMode::AKM_RISING_EDGE;
    case AutomationInputMode::falling_edge: return ButtonInputMode::AKM_FALLING_EDGE;
    case AutomationInputMode::none: return std::nullopt;
  }
  return {};
}
