/*
    Copyright (c) 2021 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <ds/reflect.h>

#include <memory>

#include "model/apartment.h"
#include "model/buttonattributes.h"
#include "model/device.h"
#include "structuremanipulator.h"

namespace dss::model {

bool isAkmDevice(const Device& device);

struct ErrorCode {
  bool success;
  std::string message;
};

struct ButtonManipulator {
  virtual ~ButtonManipulator() = default;
  //  virtual ErrorCode setButtonInputMode(Device& device, ButtonInputMode buttonInputMode) = 0;
  virtual ErrorCode setButtonInputMode(Device& device, std::string&& buttonInputMode) = 0;
  virtual ErrorCode setAKMInputProperty(Device& device, const AutomationInputMode automationInputMode) = 0;

}; // ButtonManipulator

std::unique_ptr<ButtonManipulator> makeButtonManipulator(Apartment& apartment,
                                                         StructureManipulator& structureManipulator);

} // namespace dss::model

DS_REFLECT_STRUCT((dss::model), ErrorCode, (success)(message));
