/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include <ds/catch/catch.h>
#include <ds/log.h>

//#include <trompeloeil.hpp>
#include "parser.h"

using namespace std::literals;

static const char* TAGS = "[dss][p1][parser]";

static ds::LogChannel dsLogChannel("dssP1ParserTest");

static std::string testData =
    "(/FLU5\\253769484_A\r\n"
    "0-0:96.1.4(50216)\r\n"
    "0-0:96.1.1(3153414733313030323938313039)\r\n"
    "0-0:1.0.0(220523151826S)\r\n"
    "1-0:1.8.1(000000.010*kWh)\r\n"
    "1-0:1.8.2(000000.684*kWh)\r\n"
    "1-0:2.8.1(000000.010*kWh)\r\n"
    "1-0:2.8.2(000000.149*kWh)\r\n"
    "0-0:96.14.0(0001)\r\n"
    "1-0:1.7.0(00.133*kW)\r\n"
    "1-0:2.7.0(00.000*kW)\r\n"
    "1-0:21.7.0(00.133*kW)\r\n"
    "1-0:41.7.0(00.000*kW)\r\n"
    "1-0:61.7.0(00.000*kW)\r\n"
    "1-0:22.7.0(00.000*kW)\r\n"
    "1-0:42.7.0(00.000*kW)\r\n"
    "1-0:62.7.0(00.000*kW)\r\n"
    "1-0:32.7.0(238.1*V)\r\n"
    "1-0:52.7.0(240.5*V)\r\n"
    "1-0:72.7.0(240.5*V)\r\n"
    "1-0:31.7.0(000.5*A)\r\n"
    "1-0:51.7.0(000.00*A)\r\n"
    "1-0:71.7.0(000.00*A)\r\n"
    "0-0:96.3.10(1)\r\n"
    "0-0:17.0.0(999.9*kW)\r\n"
    "1-0:31.4.0(999*A)\r\n"
    "0-0:96.13.0()\r\n"
    "!3F1A\r\n";

using dss::p1::MeterReading;
using dss::p1::parseFrame;

SCENARIO("parse test", TAGS) {
  WHEN("valid data") {
    CHECK(parseFrame(std::vector<char>{testData.begin(), testData.end()}) ==
          MeterReading{.meterId = "3153414733313030323938313039",
                       .timestamp = ds::toDateTime("2022-05-23T15:18:26.000Z"s),
                       .powerConsumed = 133,
                       .t1EnergyConsumed = 0.01,
                       .t1EnergyProduced = 0.01,
                       .t2EnergyConsumed = 0.684,
                       .t2EnergyProduced = 0.149});
  }
  WHEN("invalid data") {
    auto data{"random data"s};
    THEN("parsing fails") { CHECK(parseFrame(std::vector<char>{data.begin(), data.end()}) == std::nullopt); }
  }
}
