#include <boost/asio/local/stream_protocol.hpp>
#include <boost/asio/signal_set.hpp>
#include <iostream>
#include <memory>

#include "metering/open-metering-system.h"

using boost::asio::io_context;
using boost::asio::signal_set;
using boost::asio::local::stream_protocol;

constexpr auto socketPath = std::string_view("/tmp/socket-server");

class Session : public std::enable_shared_from_this<Session> {
public:
  Session(stream_protocol::socket&& socket) : m_socket(std::move(socket)){};

  void start() { do_read(); }

  void doParse() {
    auto tmp = ds::fromJson<dss::metering::P1Reading>(std::string(m_buffer.data()));
    std::cerr << "parsed:" << tmp << "\n";
  }

  void do_read() {
    m_socket.async_read_some(boost::asio::buffer(m_buffer),
                             [self = shared_from_this()](boost::system::error_code ec, std::size_t size) {
                               if (ec) {
                                 std::cerr << "read: error:" << ec << "\n";
                                 return;
                               }
                               std::cerr << std::string_view(self->m_buffer.data(), size) << "\n";
                               try {
                                 self->doParse();
                               } catch (std::exception& e) {
                                 std::cerr << e.what() << "\n";
                               }
                               // self->do_read();
                             });
  }

private:
  stream_protocol::socket m_socket;
  std::array<char, 1024> m_buffer{};
};

class SocketServer {
public:
  SocketServer(io_context& io, const std::string& path) : m_acceptor(io, stream_protocol::endpoint(path)) {
    do_accept();
  }

private:
  void do_accept() {
    std::cerr << "wait for accept\n";
    m_acceptor.async_accept([this](boost::system::error_code ec, stream_protocol::socket socket) {
      std::cerr << "accept triggered\n";
      if (!ec) {
        std::cout << "accepted socket\n";
        std::make_shared<Session>(std::move(socket))->do_read();
      }
      do_accept();
    });
  }

  stream_protocol::acceptor m_acceptor;
};

int main(int argc, char* argv[]) {
  std::cerr << dss::metering::P1Reading{.powerConsumed = 20, .powerProduced = 12} << "\n";
  std::remove(socketPath.data());
  boost::asio::io_context io{};
  SocketServer server(io, std::string(socketPath));
  signal_set signal(io, SIGINT, SIGTERM);
  signal.async_wait([&io](const boost::system::error_code ec, int signalNumber) {
    std::cerr << "singal" << signalNumber << "\n";
    io.stop();
  });
  std::cerr << "hello world\n";
  io.run();
}
