/*
    Copyright (c) 2019 digitalSTROM.org," " Zurich," " Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include <ds/catch/catch.h>
#include <ds/clock-mock.h>
#include <ds/log.h>

#include "sensoraggregator.h"

using namespace std::literals;

static ds::LogChannel dsLogChannel("groupTest");
static const char* TAGS = "[dss][group]";

using namespace dss;

class MaxAggregatedSensorValue : public AggregatedSensor {
public:
  MaxAggregatedSensorValue() : AggregatedSensor(max, SensorType::UnknownType) {}
};

class MinAggregatedSensorValue : public AggregatedSensor {
public:
  MinAggregatedSensorValue() : AggregatedSensor(min, SensorType::UnknownType) {}
};

static const Dsuid dsuid1{"0000000000000000000000000000000001"};
static const Dsuid dsuid2{"0000000000000000000000000000000002"};
static const Dsuid dsuid3{"0000000000000000000000000000000003"};

namespace {

TEST_CASE("aggregateSensorValue", TAGS) {
  SensorSourceValue sensorSourceValue;
  auto clock = ds::ClockMocker{};

  SECTION("Calculate Max Value") {
    MaxAggregatedSensorValue filter;
    sensorSourceValue = {.source = dsuid1, .ts = ds::Clock::now(), .value = 3.0};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 3.0);
    sensorSourceValue = {.source = dsuid2, .ts = ds::Clock::now(), .value = 2.0};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 3.0);
    sensorSourceValue = {.source = dsuid2, .ts = ds::Clock::now(), .value = 3.2};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 3.2);
    sensorSourceValue = {.source = dsuid2, .ts = ds::Clock::now(), .value = 3.1};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 3.1);
  }

  SECTION("Calculate Min Value") {
    MinAggregatedSensorValue filter;
    sensorSourceValue = {.source = dsuid1, .ts = ds::Clock::now(), .value = 3.0};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 3.0);
    sensorSourceValue = {.source = dsuid2, .ts = ds::Clock::now(), .value = 2.0};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 2.0);
    sensorSourceValue = {.source = dsuid3, .ts = ds::Clock::now(), .value = 2.1};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 2.0);
    sensorSourceValue = {.source = dsuid2, .ts = ds::Clock::now(), .value = 2.1};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 2.1);
  }

  SECTION("Age Sensor") {
    MaxAggregatedSensorValue filter;
    sensorSourceValue = {.source = dsuid1, .ts = ds::Clock::now(), .value = 3.0};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 3.0);

    clock.advance(60s);
    sensorSourceValue = {.source = dsuid2, .ts = ds::Clock::now(), .value = 2.0};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 3.0);

    clock.advance(1h);
    sensorSourceValue = {.source = dsuid3, .ts = ds::Clock::now(), .value = 1.0};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 2.0);

    clock.advance(60s);
    sensorSourceValue = {.source = dsuid1, .ts = ds::Clock::now(), .value = 0.1};
    filter.setValue(sensorSourceValue);
    CHECK(filter.get()->value == 1.0);
  }
}

} // namespace
