/*
    Copyright (c) 2019 digitalSTROM AG, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "structurerequesthandler.h"

#include <ds/asio/test.h>
#include <ds/catch/catch.h>
#include <ds/log.h>

#include "businterface-mock.h"
#include "model/apartment.h"
#include "model/device-mock.h"
#include "model/modelmaintenance-mock.h"

static ds::LogChannel dsLogChannel("structureRequestHandler");

using namespace dss;
using namespace trompeloeil;
using trompeloeil::_;

namespace {

static const char* TAGS = "[web][handler][structure]";

TEST_CASE("webHandlerStructureRequest", TAGS) {
  auto busInterface = BusInterfaceMock{};
  auto pApartment = makeApartment(nullptr, nullptr, &busInterface);
  auto& apartment = *pApartment;
  auto modelMaintenance = ModelMaintenanceMock{};

  StructureRequestHandler structHandler(apartment, modelMaintenance);

  auto devKM200 = createDevice(apartment, {.hardwareType = HardwareType::GE_KM200});
  auto devTKM210 = createDevice(apartment, {.hardwareType = HardwareType::GE_TKM210});
  auto devTKM230 = createDevice(apartment, {.hardwareType = HardwareType::GE_TKM230});
  auto devAKM200 = createDevice(apartment, {.hardwareType = HardwareType::SW_AKM200});
  auto devUMR200_input = createDevice(apartment, DeviceMockConf{.hardwareType = HardwareType::SW_UMR200});
  auto devUMR200_sensor = createDevice(apartment, DeviceMockConf{.hardwareType = HardwareType::SW_UMR200});
  devUMR200_sensor->setButtonInputMode(ButtonInputMode::AKM_STANDARD);
  auto devSW_TKM210 = createDevice(apartment, {.hardwareType = HardwareType::SW_TKM210});

  SECTION("Multi user group membership permitted") {
    CHECK(devKM200->hasOutput() == true);
    CHECK(structHandler.onlyOneGroupMembership(*devKM200) == false);

    CHECK(devTKM210->hasOutput() == true);
    CHECK(structHandler.onlyOneGroupMembership(*devTKM210) == false);

    CHECK(devTKM230->hasOutput() == false);
    CHECK(devTKM230->hasInput() == true);
    CHECK(structHandler.onlyOneGroupMembership(*devTKM230) == true);

    // sensors can be in multiple groups
    CHECK(devAKM200->hasOutput() == false);
    CHECK(devAKM200->hasInput() == true);
    CHECK(devAKM200->isBinaryInput() == true);
    CHECK(structHandler.onlyOneGroupMembership(*devAKM200) == false);

    CHECK(devUMR200_input->hasOutput() == false);
    CHECK(devUMR200_input->hasInput() == true);
    CHECK(devUMR200_input->isBinaryInput() == false);
    CHECK(structHandler.onlyOneGroupMembership(*devUMR200_input) == true);

    CHECK(devUMR200_sensor->hasOutput() == false);
    CHECK(devUMR200_sensor->hasInput() == true);
    CHECK(devUMR200_sensor->isBinaryInput() == true);
    CHECK(structHandler.onlyOneGroupMembership(*devUMR200_sensor) == false);

    auto devSW_SKM300 = createDevice(apartment, {.hardwareType = HardwareType::SW_SKM300});
    CHECK(structHandler.onlyOneGroupMembership(*devSW_SKM300) == true);
    devSW_SKM300->setButtonInputMode(ButtonInputMode::AKM_STANDARD);
    CHECK(structHandler.onlyOneGroupMembership(*devSW_SKM300) == false);

    CHECK(devSW_TKM210->hasOutput() == false);
    CHECK(devSW_TKM210->hasInput() == true);
    CHECK(devSW_TKM210->isBinaryInput() == false);
    CHECK(structHandler.onlyOneGroupMembership(*devSW_TKM210) == true);
  }
}

} // namespace
