
/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "webrequests.h"

#include <ds/catch/catch.h>

#include <trompeloeil.hpp>

using namespace trompeloeil;
using namespace std::literals;

using namespace dss;

static ds::LogChannel dsLogChannel("dssWebRequestsTests");
static const char* TAGS = "[api][json-api][webrequests]";

namespace {

SCENARIO("json generation", TAGS) {
  WHEN("non finite float") {
    const auto nonfinite = GENERATE(std::nan(""), std::numeric_limits<double>::infinity(), -INFINITY);
    JSONWriter json{};
    json.add("value", nonfinite);
    THEN("values are mapped to null") { CHECK(json.successJSON() == R"({"result":{"value":null},"ok":true})"); }
  }

  SECTION("converting vector<T>") {
    JSONWriter json{};
    WHEN("vector<int>") {
      json.add("values", std::vector{1, 2, 2, 3});
      THEN("vector is mapped") { CHECK(json.successJSON() == R"({"result":{"values":[1,2,2,3]},"ok":true})"); }
    }
    WHEN("vector<ModelFeatureId>") {
      json.add("values", std::vector{ModelFeatureId::pushbutton, ModelFeatureId::outconfigswitch});
      THEN("vector is mapped") {
        CHECK(json.successJSON() == R"({"result":{"values":["pushbutton","outconfigswitch"]},"ok":true})");
      }
    }
  }
}

} // namespace
